package org.gregoire.portknock.server;

import org.gregoire.portknock.util.*;
import java.net.*;
import java.io.*;

/**
 * Represents an attempted connection or active to a server.
 *
 * @author Paul Gregoire <paul%40gregoire.org>
 * @version 0.0.1
 */
public class Client implements Serializable {

    private static final long serialVersionUID = 3942529097545552670L;

    private long id;
    private InetAddress address;
    private int hashCode;

    private int sequence = 1;

    private String requestedApplication;

    /** Time at which this client can be removed from the active list */
    private long expirationTime = System.currentTimeMillis() + Server.getClientTimeout();

    public Client() {
        id = IDGenerator.getId();
        //hashCode = (int)(id ^ (id >>> 32));
    }

    public Client(InetAddress address) {
        this();
        setAddress(address);
    }

    public void incrementSequence() {
        sequence += 1;
    }

    public long getId() {
        return id;
    }

    public InetAddress getAddress() {
        return address;
    }

    public int getSequence() {
        return sequence;
    }

    public void setAddress(InetAddress address) {
        hashCode = address.hashCode();
        this.address = address;
    }

    public boolean equals(Object o) {
        //return ((Client) o).getId() == id;
        return ( (Client) o).hashCode() == hashCode;
    }

    public int hashCode() {
        return hashCode;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer("[Client: id=");
        sb.append(id);
        sb.append(", Address=");
        sb.append(address);
        sb.append(", hashCode=");
        sb.append(hashCode);
        sb.append("]");
        return sb.toString();
    }

    public String getRequestedApplication() {
        return requestedApplication;
    }

    public void setRequestedApplication(String requestedApplication) {
        this.requestedApplication = requestedApplication;
    }

    /**
     * Returns true if this clients expiration time has exceeded the current time.
     * @return boolean
     */
    public boolean isExpired() {
        return this.expirationTime >= System.currentTimeMillis();
    }

}
