package org.gregoire.util.crypto;

import java.security.*;

import org.apache.log4j.*;

/*
  This code is part of the Java Adaptive Network Client by Ian Clarke.
  It is distributed under the GNU Public Licence (GPL) version 2.  See
  http://www.gnu.org/ for further details of the GPL.
 */

/**
 * Interfaces with the Rijndael AES candidate to implement the Rijndael
 * algorithm
 */
public class Rijndael implements BlockCipher {

    private static Logger logger;

    static {
        //log for this class only
        logger = Logger.getLogger(Rijndael.class.getName());
    }

    private Object sessionKey;
    private int keysize, blocksize;

    // for Util.getCipherByName..  and yes, screw you too, java

    public Rijndael(Integer keysize) throws InvalidKeyException {
        this(keysize.intValue());
    }

    public Rijndael(int keysize) throws InvalidKeyException {
        this(keysize, 128);
    }

    public Rijndael(int keysize, int blocksize) throws InvalidKeyException {
        if (! (keysize == 128 ||
               keysize == 192 ||
               keysize == 256)) {
            throw new InvalidKeyException("Invalid keysize");
        }
        if (! (blocksize == 128 ||
               blocksize == 192 ||
               blocksize == 256)) {
            throw new InvalidKeyException("Invalid blocksize");
        }
        this.keysize = keysize;
        this.blocksize = blocksize;
    }

    public Rijndael() {
        this.keysize = 128;
        this.blocksize = 128;
    }

    public final int getBlockSize() {
        return blocksize;
    }

    public final int getKeySize() {
        return keysize;
    }

    public void initialize(byte[] key) {
        try {
            byte[] nkey = new byte[keysize >> 3];
            System.arraycopy(key, 0, nkey, 0, nkey.length);
            sessionKey = Rijndael_Algorithm.makeKey(nkey);
        } catch (InvalidKeyException e) {
            logger.error("Invalid key", e);
        }
    }

    public final void encrypt(byte[] block, byte[] result) {
        Rijndael_Algorithm.blockEncrypt(block, result, 0, sessionKey);
    }

    public final void decrypt(byte[] block, byte[] result) {
        Rijndael_Algorithm.blockDecrypt(block, result, 0, sessionKey);
    }

}
