package org.gregoire.util.crypto;

import junit.framework.*;
import java.security.*;

import org.gregoire.util.HexUtil;
import org.apache.log4j.*;

/**
 * <p>Title: jPortknock</p>
 * <p>Description: Port knock security daemon</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Gregorie.org</p>
 * @author Paul Gregoire
 * @version 3.0
 */
public class TestRijndael extends TestCase {

    protected void setUp() throws Exception {
        super.setUp();
    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }

    static void monteCarlo(int keySize) throws InvalidKeyException {
        Rijndael ctx = new Rijndael(keySize);
        int kb = keySize / 8;
        byte[] P = new byte[16], C = new byte[16],
            CL = new byte[16], KEY = new byte[kb];

        for (int i = 0; i < 400; i++) {
            System.out.println("I=" + i);
            System.out.println("KEY=" + HexUtil.bytesToHex(KEY, 0, kb));
            System.out.println("PT=" + HexUtil.bytesToHex(P, 0, 16));

            ctx.initialize(KEY);
            for (int j = 0; j < 10000; j++) {
                System.arraycopy(C, 0, CL, 0, C.length);
                ctx.encrypt(P, C);
                System.arraycopy(C, 0, P, 0, P.length);
            }
            System.out.println("CT=" + HexUtil.bytesToHex(C, 0, 16));

            for (int x = 0; x < kb; x++) {
                if (keySize == 192) {
                    if (x < 8) {
                        KEY[x] ^= CL[8 + x];
                    } else {
                        KEY[x] ^= C[x - 8];
                    }
                } else if (keySize == 256) {
                    if (x < 16) {
                        KEY[x] ^= CL[x];
                    } else {
                        KEY[x] ^= C[x - 16];
                    }
                } else {
                    KEY[x] ^= C[x];
                }
            }

            if (keySize == 192) {
                for (int x = 0; x < 8; x++) {
                    KEY[x + 16] ^= CL[x + 8];
                }
            } else if (keySize == 256) {
                for (int x = 0; x < 16; x++) {
                    KEY[x + 16] ^= CL[x];
                }
            }
            System.out.println();
        }
    }

    public void testMain() throws InvalidKeyException {
        // Perform the Monte Carlo test
        System.out.println("KEYSIZE=128\n");
        monteCarlo(128);
        System.out.println("=========================\n");
        System.out.println("KEYSIZE=192\n");
        monteCarlo(192);
        System.out.println("=========================\n");
        System.out.println("KEYSIZE=256\n");
        monteCarlo(256);
    }

}
